@@ -216,19 +216,13 @@ impl Client {
     /// ```
     pub async fn public_key(&self) -> PublicKey {
         let id = self.keypair().await;
-
         id.public_key()
     }
 
     /// Send a Query to the network and await a response
     async fn send_query(&self, query: Query) -> Result<QueryResult, Error> {
-        // `sign` should be false for GETs on published data, true otherwise.
-
         debug!("Sending QueryRequest: {:?}", query);
-
-        let message = self.create_query_message(query).await?;
-
-        self.session.send_query(&message).await
+        self.session.send_query(query).await
     }
 
     // Build and sign Cmd Message Envelope
@@ -242,17 +236,6 @@ impl Client {
         })
     }
 
-    // Build a Query
-    pub(crate) async fn create_query_message(&self, msg_contents: Query) -> Result<Message, Error> {
-        let id = MessageId::new();
-        trace!("Creating query message with id : {:?}", id);
-
-        Ok(Message::Query {
-            query: msg_contents,
-            id,
-        })
-    }
-
     // Private helper to obtain payment proof for a data command, send it to the network,
     // and also apply the payment to local replica actor.
     async fn pay_and_send_data_command(&self, cmd: DataCmd) -> Result<(), Error> {
